
:- discontiguous(latex/1).


/* ================================================================== ATOMICS */

latex( X ) :-
   nonvar(X),
   atomic(X),
   member([Y,Replacement],[
      [alpha,alpha],[beta,beta],[gamma,gamma], [delta,delta],
      [epsilon,varepsilon],[zeta,zeta],[eta,eta],[theta,theta], [iota,iota],
      [kappa,kappa],[lambda,lambda],[my,mu],[ny,nu],[xi,xi],[pi,pi],[rho, rho],
      [sigma,sigma],[tau,tau],[ypsilon,upsilon],[phi,varphi],[chi,chi],
      [psi,psi],[chi,chi],[psi,psi],[omega,omega]]),
   atom_concat(Y, Index, X),
   write('\\'),
   write(Replacement),
   (Index \= ''
   -> write('_{'),
      latex(Index),
      write('} ')
   ;  true).

latex( X ) :-
   nonvar(X),
   atomic(X),
   write(X).

/* =============================================== BASIC ARITHMETIC FUNCTIONS */

latex(A+B):-
   latex(A),
   write('+'),
   latex(B).
latex(-A):-
   write('-'),
   latex(A).
latex(A-B):-
   write('\\left( '),
   latex(A),
   write('-'),
   latex(B),
   write('\\right) ').
latex(A*B):-
   ((A=..[+|_] ; A=..[-|_])
   -> write('\\left( '),
      latex(A),
      write('\\right) ')
   ;  latex(A)),
   write(' \\cdot '),
   ((B=..[+|_] ; B=..[-|_])
   -> write('\\left( '),
      latex(B),
      write('\\right) ')
   ;  latex(B)).
latex(A/B):-
   write('\\frac{'),
   latex(A),
   write('}{'),
   latex(B),
   write('} ').

latex(sum(I,Lower,Upper,Term)):-
   write('\\sum_{'),
   write(I),
   write('='),
   latex(Lower),
   write('}^{'),
   latex(Upper),
   write('} '),
   ((Term=..[+|_] ; Term=..[-|_])
   -> write('\\left( '),
      latex(Term),
      write('\\right) ')
   ;  latex(Term)).

latex(product(I,Lower,Upper,Term)):-
   write('\\prod_{'),
   write(I),
   write('='),
   latex(Lower),
   write('}^{'),
   latex(Upper),
   write('} '),
   ((Term=..[+|_] ; Term=..[-|_] ; Term=..[*|_] ; Term=..[/|_])             /**/
   -> write('\\left( '),
      latex(Term),
      write('\\right) ')
   ;  latex(Term)).

/* ================================== TRIGONOMETRIC AND EXPONENTIAL FUNCTIONS */

latex(exp(X)):-
   write('e^{'),
   latex(X),
   write('}').
latex(A^(-B)):-
   latex(1/A^B).
latex(A^(B/C)):-
   write('\\sqrt'),
   (C \= 2
   -> write('['),
      latex(C),
      write(']')
   ; true),
   write('{'),
   latex(A),
   write('} '),
   (B \= 1
   -> write('^{'),
      latex(B),
      write('} ')
   ; true).
latex(A^B):-
   ((A=..[+|_] ; A=..[*|_] ; A=..[-|_] ; A=..[ / |_])                       /**/
   -> write('\\left( '),
      latex(A),
      write('\\right) ')
   ;  latex(A)
   ),
   write('^{'),
   latex(B),
   write('} ').

latex(log(A,Y)):-
   write('\\log_{ '),
   latex(A),
   write('}\\left( '),
   latex(Y),
   write(' \\right) ').
latex(X):-
   X=..[Name|[Param]],
   member(Name,[sin, cos, tan, cot, arcsin, arccos, arctan, ln]),
   write('\\'),
   write(Name),
   write('\\left( '),
   latex(Param),
   write('\\right) ').

/* ==================================================================== LISTS */

list_to_latex([X|R]):-
   latex(X),
   (R \= [] -> write(','), list_to_latex(R)).
latex( [] ) :-
   write('\left[ \right] ').
latex( [X|R] ) :-
   write('\left[ '),
   list_to_latex([X|R]),
   write('\right] ').

/* ==================================================================== TREES */

tree_to_latex( nil ).
tree_to_latex( node(X,nil,nil) ) :-
   write('{'),
   latex(X),
   write('}').
tree_to_latex( node(X,L,R) ) :-
   (L=nil -> R\=nil ; true),
   write('[.{'),
   latex(X),
   write('} '),
   tree_to_latex( L ),
   write(' '),
   tree_to_latex( R ),
   write(' ]').
latex( node(X,L,R) ) :-
   write('\\Tree '),
   tree_to_latex(node(X,L,R)).

/* ================================================================ AVL-TREES */

avl_tree_to_latex( nil ).
avl_tree_to_latex( node(X,Bal,nil,nil) ) :-
   write('{'),
   latex(X),
   write(' ('),
   write(Bal),
   write(')}').
avl_tree_to_latex( node(X,Bal,L,R) ) :-
   (L=nil -> R\=nil ; true),
   write('[.{'),
   latex(X),
   write(' ('),
   write(Bal),
   write(')} '),
   avl_tree_to_latex( L ),
   write(' '),
   avl_tree_to_latex( R ),
   write(' ]').
latex( node(X,Bal,L,R) ) :-
   write('\\Tree '),
   avl_tree_to_latex(node(X,Bal,L,R)).

/* ========================================================= RELATIONS, ROUND */

latex(A<B):-
   latex(A),
   write('<'),
   latex(B).
latex(A>B):-
   latex(A),
   write('>'),
   latex(B).
latex(A=<B):-
   latex(A),
   write('\\le '),
   latex(B).
latex(A>=B):-
   latex(A),
   write('\\ge '),
   latex(B).
latex(A=B):-
   latex(A),
   write('='),
   latex(B).
latex(A\=B):-
   latex(A),
   write('\\not= '),
   latex(B).
latex(A=:=B):-
   latex(A),
   write('='),
   latex(B).
latex(A=\=B):-
   latex(A),
   write('\\not= '),
   latex(B).
latex(ceil(A)):-
   write('\\lceil '),
   latex(A),
   write(' \\rceil ').
latex(ceiling(A)):-
   latex(ceil(A)).
latex(floor(A)):-
   write('\\lfloor '),
   latex(A),
   write(' \\rfloor ').

latex(abs(A)):-
   write('\\Vert '),
   latex(A),
   write(' \\Vert ').

/* ============================================= PROPOSITIONAL AND FO - LOGIC */

latex(( A,B )):-
   latex(A),
   write('\\wedge '),
   latex(B).
latex(( A;B )):-
   latex(A),
   write('\\vee '),
   latex(B).
latex( \+(A) ):-
   write('\\neg '),
   latex(A).
latex(( A->B )):-
   latex(A),
   write('\\rightarrow '),
   latex(B).

latex(forall(X,Y)):-
   write('\\forall '),
   latex(X),
   write('.'),
   latex(Y).

latex(exists(X,Y)):-
   write('\\exists '),
   latex(X),
   write('.'),
   latex(Y).

latex(member(X,Y)):-
   latex(X),
   write('\\in '),
   latex(Y).

/* =================================================================== GRAPHS */

latex(graph(V,E)):-
   latex(mixed_graph(V,E,[])).
latex(digraph(V,E)):-
   latex(mixed_graph(V,[],B)).
/*
latex(mixed_graph(V,E,B)):-
   writeln('% remember to include \\usepackage{gastex} and put gastex.sty from'),
   writeln('%     http://www.lsv.ens-cachan.fr/~gastin/gastex/gastex.sty'),
   writeln('% in the same directory as your project!'),nl,

   writeln('\\begin{picture}(85,33)(-20,-28)'),
   forall((member([_ID,Label,X,Y],Pos,V)),(
      writeln('   \\gasset{Nadjust=w,Nadjustdist=2,Nh=6,Nmr=1}'),
      write('   \\node(X'),write(Pos),write(')('),write(X), write(','), write(Y),
      write('){'),latex(Label),writeln('}'),
   )),
   nl,

   forall((member([From,To,Label],E),member([From|_],PosF,V),member([To|_],PosT,V),PosF=<PosT),(
      write('   \\drawedge(X'),write(PosF),write(',X'),write(PosT),write('){'),
      latex(Label),writeln('}')
   )),
   forall((member([From,To,Label],B),member([From|_],PosF,V),member([To|_],PosT,V)),(
      write('    \\drawarrow x'),write(PosF),write('{dir(30)}..{dir(-30)}x'),write(PosT),writeln(';')
   )),
   nl,
   writeln('   \drawedge(A,B){req1:=true}'),
   writeln('   \drawedge(B,C){turn:=2}'),
   writeln('   \drawedge[syo=-1,eyo=-1](C,D){turn=1?}'),
   writeln('   \drawedge[syo=1,eyo=1,ELside=r](C,D){req2=false?}'),
   writeln('   \drawedge(D,A){req1:=false}'),
   writeln('\\end{picture}'),
*/

/* ======================================================= GRAPHS TO METAPOST */

metapost(graph(V,E)):-
   metapost(mixed_graph(V,E,[])).
metapost(digraph(V,B)):-
   metapost(mixed_graph(V,[],B)).
metapost(mixed_graph(V,E,B)):-
   writeln('transform pagecoords;'),
   writeln('pagecoords:=identity scaled 100mm shifted (100mm,150mm);'),
   nl,

   writeln('beginfig(1);'),
   writeln('    pair x[];'),
   forall(member([_ID,_Label,X,Y],Pos,V),(
      write('    x'),write(Pos),write('=('),write(X),write(','),write(Y),writeln(');')
   )),
   nl,

   writeln('    % set the "dir(+-30)" parameters to dir(0) for straight lines'),
   writeln('    % append "dashed evenly" to get dashed lines'),
   forall((member([From,To],E),member([From|_],PosF,V),member([To|_],PosT,V),PosF=<PosT),(
      write('    draw x'),write(PosF),write('{dir(30)}..{dir(-30)}x'),write(PosT),writeln(';')
   )),
   forall((member([From,To],B),member([From|_],PosF,V),member([To|_],PosT,V)),(
      write('    drawarrow x'),write(PosF),write('{dir(30)}..{dir(-30)}x'),write(PosT),writeln(';')
   )),
   nl,

   writeln('    % increase the value of "scaled" to get bigger nodes'),
   length(V,L),
   write('    for i=0 upto '),write(L),writeln(':'),
   writeln('        drawdot x[i] withpen pencircle scaled 2;'),
   writeln('    endfor'),
   nl,

   writeln('    % ulft means the text will be on the Upper LeFT of the node'),
   writeln('    % other possibilities are:'),
   writeln('    % top (top), urt (upper right), rt (right), lrt (lower right),'),
   writeln('    % bot (bottom), llft (lower left), lft (left), ulft (upper left)'),
   forall(member([_ID,Label,_X,_Y],Pos,V),(
      write('    label.ulft("'),latex(Label),write('", x'),write(Pos),writeln(');')
   )),
   nl,
   writeln('endfig;'),
   writeln('bye').
